/***************************************************************************
 *
 * MODULE:	r.support.cats (command line)
 * AUTHOR(S):	Glynn Clements
 * PURPOSE:	Set the categories for a raster map.
 * 
 * COPYRIGHT:	(C) 2007 Glynn Clements
 *
 *		This program is free software under the GPL (>=v2)
 *		Read the file COPYING that comes with GRASS for details.
 ****************************************************************************
 */

#include <string.h>
#include <stdlib.h>
#include <grass/gis.h>
#include <grass/glocale.h>

int main (int argc, char *argv[])
{
	struct Categories cats;
	FILE *fp;
	char *title;
	char *name;
	char *mapset;

	struct GModule *module;
	struct {
		struct Option *map, *title, *file;
	} parm;

	G_gisinit (argv[0]);

	module = G_define_module();
	module->keywords = _("raster");
	module->description =
		_("Sets the categories for a raster map.");

	/* parameters */

	parm.map = G_define_option();
	parm.map->key		= "map";
	parm.map->key_desc	= "name";
	parm.map->required	= YES;
	parm.map->multiple	= NO;
	parm.map->type		= TYPE_STRING;
	parm.map->gisprompt	= "old,cell,raster";
	parm.map->description	= _("Raster map to set categories for");

	parm.file = G_define_option();
	parm.file->key		= "file";
	parm.file->required	= YES;
	parm.file->description	= _("Path to categories file");
	parm.file->gisprompt	= "old_file,file,input";

	parm.title = G_define_option();
	parm.title->key		= "title";
	parm.title->description	= _("Categories title");

	if (G_parser(argc,argv))
		exit (EXIT_FAILURE);

	name = parm.map->answer;

	mapset = G_find_cell2(name, G_mapset());
	if (!mapset)
		G_fatal_error(_("raster map <%s> not found"), name);

	title = parm.title->answer;
	if (!title)
	{
		char buf[1024];
		sprintf(buf, "Categories for %s", name);
		title = G_store(buf);
	}

	fp = fopen(parm.file->answer, "r");
	if (!fp)
		G_fatal_error(_("unable to open input file: %s"), parm.file->answer);

	G_init_cats(0, title, &cats);

	for (;;)
	{
		char buf[1024], label[1024];
		DCELL d1, d2;

		if (!G_getl2(buf, sizeof(buf), fp))
			break;

		if (sscanf(buf, "%lf:%lf:%[^\n]", &d1, &d2, label) == 3)
			G_set_d_raster_cat(&d1, &d2, label, &cats);
		else if (sscanf(buf, "%lf:%[^\n]", &d1, label) == 2)
			G_set_d_raster_cat(&d1, &d1, label, &cats);
	}

	if (G_write_cats(parm.map->answer, &cats) < 0)
		G_fatal_error(_("unable to update categories"));

	G_done_msg(" ");

	return 0;
}

